Red [
	title: "Red Pretty Printer"
	notes: "Derived from http://www.rebol.net/cookbook/recipes/0042.html"
]
;-- by Gregg Irwin
;-- modified by Mike Parr to use a command-line arg for file name, and to avoid
;-- overwriting if the prettifying cannot be done properly.
                                             
;-----------------------------------------------------------
non-white-length: function [
    "removes all white space, returns length of resulting string"
    the-script [string!]
    ][
	;-- Reason for use:
	;--   if too many ], clean-script can finish early; result omits code after the final matching ]'
	;--   if too few ], result is OK.
	;--   non-white-length is used to see if significant chars have been lost from the output.
	;--    if new < old, then error - some non-whites have been omitted at end, so don't update the file.
    no-white: copy the-script
	replace/all no-white " " ""
	replace/all no-white tab ""
	replace/all no-white crlf ""
	replace/all no-white newline ""
	return length? no-white
]
;---------------------------------------------------------------

output: none	; output text
indent: copy ""	; indentation chars

emit-val: func [val] [append output val]

emit-space: func [pos] [
	either newline = last output [emit-val indent] [
		if not any [find "[(" last output  find ")]" first pos] [emit-val space]
	]
]


emit-part: func [from to] [
	emit-space from
	emit-val copy/part from to
]

clean-script: func [
	"Returns new script text with standardized layout."
	script "Original Script text"
	/local blk-rule str new
][
	output: make string! length? script
	clear indent
	parse script blk-rule: [
		some [
			str:	; Get position for emitting
			[crlf | newline] (emit-val newline)
			| #";" [thru newline | to end] new: (emit-part str new)
			; Emit the opening bracket/paren we just found, update the
			; indent level, and parse into the block/paren.
			| [#"[" | #"("] (emit-part str 1  append indent tab) blk-rule
			| [#"]" | #")"] (remove indent    emit-part str 1) break
			| [space | tab] ; Do nothing. Emit-part adds spaces for us.
			| skip (value: load/next str 'new  emit-part str new) :new
		]
	]
	; We may have added a leading space before the first value we found,
	; and we don't want that in the output.
	trim/head output
]

;-------------------------------------------------------------
write %pp-status.txt ""    ;will be non ""  if pp cannot be done safely
arg1:   to file! system/script/args
;-- nb sys/args gets all args in 1 string, quotes""  not significant (nb windows users!)
in-text: read arg1
orig-non-whites: non-white-length in-text 
result: clean-script in-text
new-non-whites: non-white-length result

either new-non-whites < orig-non-whites [
print "error - too many ] in file"
	pp-ms: "File could not be indented safely. It contains too many closing ] characters."
	write %pp-status.txt pp-ms
] [
	write arg1 result    ;update with pp version
]

